@@ -2,12 +2,13 @@ import collections
 import logging
 
 from bidict import namedbidict
+import braintree
 
 from .compat import getcallargs
 
 logger = logging.getLogger(__name__)
 
-IDMap = namedbidict('IDMap', 'fake_ids', 'real_ids')
+IDMap = namedbidict('IDMap', 'fake_id', 'real_id')
 
 
 def ensure_state_is_init(f):
@@ -27,14 +28,14 @@ def ensure_state_is_init(f):
     return wrapper
 
 
-def clear_old_creation_ids(state, call_params):
+def clear_old_creation_ids(state, call_params, options):
     # Used as a start_hook in appcode entry points (ie, not __init__)
     # to ensure that old state doesn't stick around.
     state['last_fake_ids'] = {}
 
 
 @ensure_state_is_init
-def convert_to_real_id(params, schema_params, key, resource_id, state):
+def convert_to_real_id(params, schema_params, key, resource_id, state, options):
     fake_id = params[key]
     id_maps = state['id_maps']
 
@@ -43,29 +44,34 @@ def convert_to_real_id(params, schema_params, key, resource_id, state):
     # to replace it, everything still proceeds normally.
     real_id = fake_id
     try:
-        real_id = id_maps[resource_id.bt_class].fake_ids[fake_id]
+        real_id = id_maps[resource_id.bt_class].fake_id_for[fake_id]
     except KeyError:
         # This can happen in two cases:
         #   * The caller made a mistake and didn't create the resource yet.
         #   * The caller created the resource, but not through our (patched) braintree library.
-        logger.warning("The braintree id %r has not been previously stored."
-                       " Either the resource was never created,"
-                       " or it was not created through this client and namespace.", fake_id)
+        if (('strict_missing_exception' in options or
+             options.get('strict_missing', False))):
+            exception = options.get('strict_missing_exception', braintree.exceptions.NotFoundError)
+            raise exception
+        else:
+            logger.warning("The braintree id %r has not been previously stored."
+                           " Either the resource was never created,"
+                           " or it was not created through this client and namespace.", fake_id)
 
     params[key] = real_id
     logger.debug("%r --[real_id]--> %r", fake_id, params[key])
 
 
 @ensure_state_is_init
-def delete_and_store(params, schema_params, key, resource_id, state):
+def delete_and_store(params, schema_params, key, resource_id, state, options):
     provided_id = params[key]
     bt_class = resource_id.bt_class
     id_maps = state['id_maps']
 
-    if provided_id in id_maps[bt_class].fake_ids:
+    if provided_id in id_maps[bt_class].fake_id_for:
         # Properly handle duplicate creates of the same id.
         # We should pass these through, since the gateway will return an error.
-        params[key] = id_maps[bt_class].fake_ids[provided_id]
+        params[key] = id_maps[bt_class].fake_id_for[provided_id]
         logger.debug("would have deleted, but %r has been used in a prior creation", provided_id)
     else:
         # We need to know which id the client expects the response to be mapped to.
@@ -76,13 +82,13 @@ def delete_and_store(params, schema_params, key, resource_id, state):
 
 
 @ensure_state_is_init
-def convert_to_fake_id(params, schema_params, key, resource_id, state):
+def convert_to_fake_id(params, schema_params, key, resource_id, state, options):
     real_id = params[key]
     id_maps = state['id_maps']
     bt_class = resource_id.bt_class
     last_fake_ids = state['last_fake_ids']
 
-    if real_id not in id_maps[bt_class].real_ids:
+    if real_id not in id_maps[bt_class].real_id_for:
         # We need to update our mapping.
         # This condition also prevents us from updating existing mappings,
         # which we'd want to change to support id updates.
@@ -99,8 +105,8 @@ def convert_to_fake_id(params, schema_params, key, resource_id, state):
             #       operation is incredibly slim.
             fake_id = real_id
 
-        id_maps[bt_class].fake_ids[fake_id] = real_id
+        id_maps[bt_class].fake_id_for[fake_id] = real_id
         logger.debug('mapping updated: fake_id %r == %r', fake_id, real_id)
 
-    params[key] = id_maps[bt_class].real_ids[real_id]
+    params[key] = id_maps[bt_class].real_id_for[real_id]
     logger.debug("%r <--[fake_id]-- %r", params[key], real_id)
