@@ -1,7 +1,11 @@
+from builtins import str
+from builtins import object
 import copy
 import functools
 import logging
 
+from mock import patch
+
 from .compat import getcallargs
 from .schemas import ResourceId
 
@@ -11,16 +15,18 @@ logger = logging.getLogger(__name__)
 class PatchedMethod(object):
     """Instances of this callable replace braintree methods."""
 
-    def __init__(self, method, state, call_schema):
+    def __init__(self, method, state, call_schema, options):
         """
         :param method: a staticmethod or instance method
         :param state: the state dictionary to provide to actions
         :param call_schema
+        :param options: dictionary with arbitrary contents passed through to actions
         """
 
         self.method = method
         self.state = state
         self.call_schema = call_schema
+        self.options = options
 
     def __call__(self, *args, **kwargs):
         named_args = getcallargs(self.method, *args, **kwargs)
@@ -33,7 +39,7 @@ class PatchedMethod(object):
             named_args_copy[key] = copy.deepcopy(named_args_copy[key])
 
         if self.call_schema.start_hook is not None:
-            self.call_schema.start_hook(self.state, named_args_copy)
+            self.call_schema.start_hook(self.state, named_args_copy, self.options)
 
         self._apply_param_actions(named_args_copy, self.call_schema.params)
 
@@ -48,7 +54,7 @@ class PatchedMethod(object):
     def _apply_param_actions(self, params, schema_params):
         """Traverse a schema and perform the updates it describes to params."""
 
-        for key, val in schema_params.items():
+        for key, val in list(schema_params.items()):
             if key not in params:
                 continue
 
@@ -62,7 +68,7 @@ class PatchedMethod(object):
                 params[key] = str(params[key])
 
                 resource_id.action(params, schema_params, key,
-                                   resource_id, self.state)
+                                   resource_id, self.state, self.options)
             else:
                 logger.error("Invalid value in schema params: %r. schema_params: %r and params: %r",
                              val, schema_params, params)
@@ -77,14 +83,19 @@ class PatchedMethod(object):
 
 
 class SchemaPatcher(object):
-    def __init__(self):
+    def __init__(self, options):
         self._action_state = {}
+        self.options = options
+
+    def create_patchers(self, call_schemas):
+        patchers = []
 
-    def apply_patches(self, call_schemas):
         for call_schema in call_schemas:
             bt_class = call_schema.bt_class
             original_method = getattr(bt_class, call_schema.method_name)
 
-            replacement = PatchedMethod(original_method, self._action_state, call_schema)
+            replacement = PatchedMethod(original_method, self._action_state,
+                                        call_schema, self.options)
+            patchers.append(patch.object(bt_class, call_schema.method_name, replacement))
 
-            setattr(bt_class, call_schema.method_name, replacement)
+        return patchers
